///////////////////////////////////////////////////////////////////////////////////////////
//                                                   41 Post                             //
// Android: Creating a button to load images from a remote server                        //
// Created by DimasTheDriver in 28/Jan/2011                                      		 //
// Availiable at:       http://www.41post.com/?p=2790                           		 //
///////////////////////////////////////////////////////////////////////////////////////////

package fortyonepost.com.lri;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import android.app.Activity;
import android.app.ProgressDialog;
import android.view.View.OnClickListener;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;

/*Implement the Runnable interface for creating the thread and
 * the OnClickListener to start the thread inside the button*/ 
public class RemoteImagesExample extends Activity implements Runnable, OnClickListener
{
	//the handler object is created to update the UI thread from another one
	private Handler handler = new Handler();
	//a variable to store the downloaded Bitmap
	public Bitmap downloadedBitmap;
	//create a variable for the button
	private Button btn_download;
	//create a variable for the image in the View
	private ImageView img_downloaded;
	//create a progress dialog
	private ProgressDialog dialog;
	//a string array for the file name
	private String[] filepath;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) 
    {
        super.onCreate(savedInstanceState);
        
		//set this view as the content of this Activity
        setContentView(R.layout.main);  
        
        //create a dialog
		dialog = new ProgressDialog(this);
		//set the title of the dialog
        dialog.setTitle("Downloading Image");
        //Set if the dialog can be skipped
        dialog.setCancelable(false);
        //Set if the dialog doesn't have a estimated time to be dismissed
        dialog.setIndeterminate(true);
       
        //get the button from the main.xml layout file
        btn_download = (Button) findViewById(R.id.btn_download);
		
        //get the image from the main.xml layout file
		img_downloaded = (ImageView) findViewById(R.id.img_dowloaded);
        
		//set the button's OnClickListenter - which will be the one implemented on this class
		btn_download.setOnClickListener(this);
    }
    
    //Since this class implements the OnClickListener, the OnClick method must be overridden
	@Override
	public void onClick(View v)
	{
		//displays the dialog
		dialog.show();
		//starts a new thread that will execute all the code inside the run() method.
		new Thread(this).start();
	}
    
    //This method is responsible for downloading the image from a remote location
	private Bitmap DownloadBMP(String url) throws IOException
	{
		//create a URL object from the passed string
		URL location = new URL(url);
		
		/*Get the name of the file and its path, and break it into different parts.
		 *Store each of these parts as elements in the filepath array.*/
		filepath = location.getFile().split("\u002F");
		
		/*The last element of the filepath array will be the name of the file.
		 *Display the name in the progress dialog.*/
		dialog.setMessage("Downloading " + filepath[filepath.length-1]);
		
		//create a InputStream object, to read data from a remote location
		InputStream input_s = location.openStream(); 
		
		//use the BitmapFactory to decode the downloaded stream into a Bitmap
		Bitmap returnedBMP = BitmapFactory.decodeStream(input_s);
		
		//close the InputStream
		input_s.close();
		
		//returns the downloaded bitmap
		return returnedBMP;
	}

	//this method must be overridden, as we are implementing the Runnable interface
	@Override
	public void run() 
	{
		//Download the image
		try 
		{
			downloadedBitmap = DownloadBMP("http://img41.imageshack.us/img41/426/bricks.gif");
		} 
		catch (IOException e) 
		{
			//If the image couldn't be downloaded, use the standard 'image not found' bitmap
			downloadedBitmap = BitmapFactory.decodeResource(getResources(), R.drawable.imagenotfound);
		}
		
		//update the canvas from this thread using the Handler object
		handler.post(new Runnable() 
		{
			@Override
			public void run() 
			{
				//set the image on the View to display the downloaded Bitmap.
				img_downloaded.setImageBitmap(downloadedBitmap);
				
				//close the progress dialog.
				dialog.dismiss();
			}
		});	
	}
}